export declare const ERR_BUFFER_NOT_FOUND = "BUFFER NOT FOUND";
export declare const ERR_BUFFER_ALREADY_EXISTS = "BUFFER ALREADY EXISTS";
export declare const FLUSH_BUFFER_EVENT_NAME = "BUFFER:flush";
export declare const CLEAN_BUFFER_EVENT_NAME = "BUFFER:clean";
export declare class BufferedEventEmitter {
    private debug;
    private map;
    private bufferedMessages;
    private ttl;
    private maintenanceChance;
    static FLUSH_BUFFER_EVENT_NAME: string;
    static CLEAN_BUFFER_EVENT_NAME: string;
    constructor(options?: BufferedEventEmitterOptions);
    /**
     * Enable/disable debug mode (console.log everywhere ;))
     */
    setDebugMode(value: boolean): this;
    /**
     * Creates a named buffer
     */
    createBuffer(bufferId: number | string, context?: any): this;
    /**
     * Add and event to the buffer.
     * Will store the event until the buffer is flushed
     */
    emitBuffered(bufferId: number | string, eventName: string, message: any): this;
    /**
     * Flushes the buffer, calling the registered handlers for all events
     * Maintenance will try to remove it after a few moments.
     */
    flush(bufferId: string | number): this;
    /**
     * Remove all data from the buffer. Just the global clean buffer event will be emitted
     */
    cleanBuffer(bufferId: number | string): this;
    /**
     * Add an event listener
     * @returns {Function} the "unsubscriber". Call this function to unsubscribe this event (or use the unsubscribe method)
     */
    subscribe(eventName: string, fn: Function): Function;
    /**
     * @see subscribe
     * Add an event listener to multiple event aht the same time
     *
     * @param {String[]} eventNames Event's names
     * @param {Function} fn Handler
     * @return {Function} Unsubscriber for all events
     * @see EventManager.subscribe
     */
    subscribeMultiple(eventNames: string[], fn: Function): Function;
    /**
     * Removes an event listener from an event
     *
     * @param {string} eventName Event's name
     * @param {Function} fn Handler to remove
     */
    unsubscribe(eventName: string, fn: Function): this;
    /**
     * @see unsubscribe
     * Removes the event listener from multiple events
     */
    unsubscribeMultiple(eventNames: string[], fn: Function): this;
    /**
     * Removes all event listeners from the given events
     */
    unsubscribeAll(eventNames: string[]): this;
    /**
     * Trigger an event. Will send all arguments after eventName to the existent
     * event listeners
     *
     * @param {String} eventName Event's name
     * @param {...*} args
     * @memberOf EventManager
     */
    emit(eventName: string, ...args: any[]): this;
    /**
     * Checks if the buffer exists
     * @param id
     */
    bufferExists(id: number | string): boolean;
    getBuffer(id: number | string): BufferedEventEmitterBuffer;
    private validateBufferExists;
    /**
     * This method returns a buffer. the difference between this method and the getBuffer is that this one performs a copy
     * of the buffer before returning it if shouldCopy is true. Otherwise it returns the original buffer.
     * For performance reasons, the original buffer is returned by default and only should be used internally. For external
     * usage, use the getBuffer method.
     *
     * @param bufferId
     * @param shouldCopy
     * @private
     */
    private internalGetBuffer;
    /**
     * Checks if maintenance should be done and do it
     */
    private checkMaintenance;
    private maintenance;
    private setTTL;
    private setMaintenanceChance;
    private log;
}
/** Options to configure the EventEmitter */
export interface BufferedEventEmitterOptions {
    /**
     * TimeToLive: Time in seconds that the buffer can exists without activity.
     * After this period it isn't valid anymore and can eventually be removed.
     *
     * Example: 10
     */
    ttl?: number;
    /**
     * Every call to some methods will have a chance to run the maintenance, witch
     * is the process of remove expired buffers not flushed. This number define
     * the chance of this process happen on a method call (percent);
     * Example: 10 => 10% chance of running maintenance
     */
    maintenanceChance?: number;
}
/**
 * The buffer
 */
export interface BufferedEventEmitterBuffer {
    /** Unique id of the buffer */
    id: number | string;
    /**
     * Extra information about the buffer.
     * You can put anything you want in here.
     */
    context: any;
    /**
     * Creation date of the buffer
     */
    created: Date;
    /**
     * Date of the last change of the buffer - usually the creation date of the last
     * event.
     */
    lastActivity: Date;
    /**
     * Store emitted events.
     */
    events: BufferedEventEmitterBufferEvents;
}
/**
 *
 */
export interface BufferedEventEmitterBufferHash {
    [propName: string | number]: BufferedEventEmitterBuffer;
}
/**
 *
 */
export interface BufferedEventEmitterBufferEvents {
    [propName: string]: any[];
}
