"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BufferedEventEmitter = exports.CLEAN_BUFFER_EVENT_NAME = exports.FLUSH_BUFFER_EVENT_NAME = exports.ERR_BUFFER_ALREADY_EXISTS = exports.ERR_BUFFER_NOT_FOUND = void 0;
exports.ERR_BUFFER_NOT_FOUND = 'BUFFER NOT FOUND';
exports.ERR_BUFFER_ALREADY_EXISTS = 'BUFFER ALREADY EXISTS';
exports.FLUSH_BUFFER_EVENT_NAME = 'BUFFER:flush';
exports.CLEAN_BUFFER_EVENT_NAME = 'BUFFER:clean';
const BUFFER_RETENTION_PERIOD_SECONDS = 1;
const MAINTENANCE_CHANCE = 100;
function copy(object) {
    return JSON.parse(JSON.stringify(object));
}
function isObject(object) {
    return typeof object === 'object';
}
class BufferedEventEmitter {
    constructor(options = {}) {
        var _a, _b;
        this.debug = false;
        this.map = {};
        this.bufferedMessages = {};
        this.setTTL((_a = options === null || options === void 0 ? void 0 : options.ttl) !== null && _a !== void 0 ? _a : BUFFER_RETENTION_PERIOD_SECONDS)
            .setMaintenanceChance((_b = options === null || options === void 0 ? void 0 : options.maintenanceChance) !== null && _b !== void 0 ? _b : MAINTENANCE_CHANCE);
    }
    /**
     * Enable/disable debug mode (console.log everywhere ;))
     */
    setDebugMode(value) {
        this.debug = value;
        return this;
    }
    /**
     * Creates a named buffer
     */
    createBuffer(bufferId, context = {}) {
        this.log(`Trying to create buffer ${bufferId}`);
        this.checkMaintenance();
        if (bufferId in this.bufferedMessages)
            throw new Error(exports.ERR_BUFFER_ALREADY_EXISTS);
        const now = new Date();
        this.bufferedMessages[bufferId] = {
            id: bufferId,
            context: isObject(context) ? copy(context) : context,
            created: now,
            lastActivity: now,
            events: {}
        };
        this.log(`Buffer ${bufferId} created`);
        return this;
    }
    /**
     * Add and event to the buffer.
     * Will store the event until the buffer is flushed
     */
    emitBuffered(bufferId, eventName, message) {
        this.log(`Emitting buffered event ${eventName} on buffer ${bufferId}`);
        const buffer = this.internalGetBuffer(bufferId, false);
        if (!(eventName in buffer.events)) {
            this.log(`Creating event ${eventName} on buffer ${bufferId}`);
            buffer.events[eventName] = [];
        }
        const args = Array.prototype.slice.call(arguments, 2);
        buffer.events[eventName].push(args);
        buffer.lastActivity = new Date();
        this.log(`Buffer ${bufferId} updated`);
        return this;
    }
    /**
     * Flushes the buffer, calling the registered handlers for all events
     * Maintenance will try to remove it after a few moments.
     */
    flush(bufferId) {
        this.log(`Trying to flush buffer ${bufferId}`);
        const buffer = this.internalGetBuffer(bufferId, true);
        const context = isObject(buffer.context) ? copy(buffer.context) : buffer.context;
        if (exports.FLUSH_BUFFER_EVENT_NAME in this.map) {
            this.log('Calling handlers for flush event');
            this.map[exports.FLUSH_BUFFER_EVENT_NAME].forEach(fn => fn(buffer.id, buffer.context, buffer.events));
        }
        this.log('Calling handlers for buffered events');
        Object.keys(buffer.events).forEach(eventName => {
            if (eventName in this.map) {
                buffer.events[eventName].forEach(event => {
                    event.push(isObject(context) ? copy(context) : context);
                    this.map[eventName].forEach(fn => fn.apply(this, event));
                });
            }
        });
        // eslint-disable-next-line @typescript-eslint/no-dynamic-delete
        delete this.bufferedMessages[bufferId];
        return this;
    }
    /**
     * Remove all data from the buffer. Just the global clean buffer event will be emitted
     */
    cleanBuffer(bufferId) {
        this.log(`Cleaning buffer ${bufferId}`);
        const buffer = this.internalGetBuffer(bufferId, true);
        if (BufferedEventEmitter.CLEAN_BUFFER_EVENT_NAME in this.map) {
            this.log('Calling clean buffer event handler');
            this.map[BufferedEventEmitter.CLEAN_BUFFER_EVENT_NAME].forEach(fn => fn(buffer.id, buffer.context, buffer.events));
        }
        // eslint-disable-next-line @typescript-eslint/no-dynamic-delete
        delete this.bufferedMessages[bufferId];
        return this;
    }
    /**
     * Add an event listener
     * @returns {Function} the "unsubscriber". Call this function to unsubscribe this event (or use the unsubscribe method)
     */
    subscribe(eventName, fn) {
        if (typeof eventName !== 'string')
            throw new Error('eventName must be string'); // NOSONAR
        if (eventName.length === 0)
            throw new Error('eventName cannot be empty');
        if (!(eventName in this.map))
            this.map[eventName] = [];
        this.map[eventName].push(fn);
        return this.unsubscribe.bind(this, eventName, fn);
    }
    /**
     * @see subscribe
     * Add an event listener to multiple event aht the same time
     *
     * @param {String[]} eventNames Event's names
     * @param {Function} fn Handler
     * @return {Function} Unsubscriber for all events
     * @see EventManager.subscribe
     */
    subscribeMultiple(eventNames, fn) {
        const unsubscribes = eventNames.map(eventName => this.subscribe(eventName, fn));
        return () => unsubscribes.forEach(unsubscribe => unsubscribe());
    }
    /**
     * Removes an event listener from an event
     *
     * @param {string} eventName Event's name
     * @param {Function} fn Handler to remove
     */
    unsubscribe(eventName, fn) {
        if (!(eventName in this.map)) {
            return this;
        }
        const index = this.map[eventName].indexOf(fn);
        if (index !== -1)
            this.map[eventName].splice(index, 1);
        return this;
    }
    /**
     * @see unsubscribe
     * Removes the event listener from multiple events
     */
    unsubscribeMultiple(eventNames, fn) {
        let i;
        const length = eventNames.length;
        for (i = 0; i < length; i++) {
            this.unsubscribe(eventNames[i], fn);
        }
        return this;
    }
    /**
     * Removes all event listeners from the given events
     */
    unsubscribeAll(eventNames) {
        // eslint-disable-next-line @typescript-eslint/no-dynamic-delete
        eventNames.forEach(name => name in this.map && delete this.map[name]);
        return this;
    }
    /**
     * Trigger an event. Will send all arguments after eventName to the existent
     * event listeners
     *
     * @param {String} eventName Event's name
     * @param {...*} args
     * @memberOf EventManager
     */
    emit(eventName, ...args) {
        this.log(`Emitting event ${eventName}`);
        if (!(eventName in this.map))
            return this;
        // make an copy of the arguments to prevent someone to change it
        this.map[eventName].forEach(fn => fn.apply(this, copy(args)));
        return this;
    }
    /**
     * Checks if the buffer exists
     * @param id
     */
    bufferExists(id) {
        return id in this.bufferedMessages;
    }
    getBuffer(id) {
        return this.internalGetBuffer(id, true);
    }
    validateBufferExists(id) {
        if (!this.bufferExists(id))
            throw new Error(exports.ERR_BUFFER_NOT_FOUND);
        return this;
    }
    /**
     * This method returns a buffer. the difference between this method and the getBuffer is that this one performs a copy
     * of the buffer before returning it if shouldCopy is true. Otherwise it returns the original buffer.
     * For performance reasons, the original buffer is returned by default and only should be used internally. For external
     * usage, use the getBuffer method.
     *
     * @param bufferId
     * @param shouldCopy
     * @private
     */
    internalGetBuffer(bufferId, shouldCopy = false) {
        this.validateBufferExists(bufferId);
        const buffer = this.bufferedMessages[bufferId];
        return shouldCopy ? copy(buffer) : buffer;
    }
    /**
     * Checks if maintenance should be done and do it
     */
    checkMaintenance() {
        this.log('Checking maintenance...');
        if (Math.random() <= this.maintenanceChance / 100) {
            try {
                this.maintenance();
            }
            catch (e) {
                console.error('Failed to run maintenance.', e);
            }
        }
        return this;
    }
    maintenance() {
        this.log('Running maintenance...');
        const now = new Date();
        Object.keys(this.bufferedMessages).forEach(id => {
            const buffer = this.internalGetBuffer(id);
            const diff = now.getTime() - buffer.lastActivity.getTime();
            const seconds = Math.abs(diff / 1000);
            if (seconds > this.ttl) {
                this.cleanBuffer(id);
            }
        });
        return this;
    }
    setTTL(ttl) {
        if (!Number.isSafeInteger(ttl))
            throw new Error('Invalid TTL: must be an integer');
        if (ttl < 1)
            throw new Error('Invalid TTL: must be greater than 0');
        this.ttl = ttl;
        return this;
    }
    setMaintenanceChance(change) {
        if (Number.isNaN(change))
            throw new Error('Invalid maintenanceChance: must be numeric');
        if (change <= 0 || change > 100)
            throw new Error('Invalid maintenanceChance: must be greater than 0 and lower than 100');
        this.maintenanceChance = change;
        return this;
    }
    log(arg) {
        this.debug && console.log(arg);
    }
}
exports.BufferedEventEmitter = BufferedEventEmitter;
BufferedEventEmitter.FLUSH_BUFFER_EVENT_NAME = exports.FLUSH_BUFFER_EVENT_NAME;
BufferedEventEmitter.CLEAN_BUFFER_EVENT_NAME = exports.CLEAN_BUFFER_EVENT_NAME;
